@@ -1,11 +1,17 @@
 from public import public as _public
 
 from flufl.lock._lockfile import (
-    AlreadyLockedError, Lock, LockError, LockState, NotLockedError, SEP,
-    TimeOutError)
+    AlreadyLockedError,
+    Lock,
+    LockError,
+    LockState,
+    NotLockedError,
+    SEP,
+    TimeOutError,
+)
 
 
-__version__ = '5.0.4'
+__version__ = '5.0.5'
 
 
 _public(
@@ -17,7 +23,7 @@ _public(
     SEP=SEP,
     TimeOutError=TimeOutError,
     __version__=__version__,
-    )
+)
 
 
 del _public
