@@ -6,13 +6,21 @@ import random
 import socket
 import logging
 
+from contextlib import suppress
+from datetime import datetime, timedelta
 from enum import Enum
+from logging import NullHandler
+from types import TracebackType
+from typing import cast, List, Optional, Tuple, Type, Union
+
 from psutil import pid_exists
 from public import public
-from typing import List, Tuple, Union, Optional
-from logging import NullHandler
-from datetime import datetime, timedelta
-from contextlib import suppress
+
+
+try:
+    from typing import Literal
+except ImportError:                                 # pragma: nocover
+    from typing_extensions import Literal  # type: ignore
 
 
 Interval = Union[timedelta, int]
@@ -29,7 +37,7 @@ MAXINT = sys.maxsize
 # Details separator; also used in calculating the claim file path.  Lock files
 # should not include this character.  We do it like this so flake8 won't
 # complain about SEP.
-SEP = ('^' if sys.platform == 'win32' else '|')
+SEP = '^' if sys.platform == 'win32' else '|'
 public(SEP=SEP)
 
 # LP: #977999 - catch both ENOENT and ESTALE.  The latter is what an NFS
@@ -73,25 +81,25 @@ class LockState(Enum):
     However it is possible to provide some insights into the state of the
     lock.  Note that the policy on what to do with this information is left
     entirely to the user of the library.
-
-    Possible states:
-
-    * There is no lock file so the lock is unlocked.
-    * We own the lock and it is fresh
-    * We own the lock but it has expired.  Another process trying
-      to acquire the lock will break it.
-    * We don't own the lock; the hostname in the details matches our
-      hostname and there is no pid running that matches pid.  Therefore,
-      the lock is stale.
-    * We don't own the lock; either our hostname does not match the
-      details, or there is a process (that's not us) with a matching pid.
-      The state of the lock is therefore unknown.
     """
+
+    #: There is no lock file so the lock is unlocked.
     unlocked = 1
+    #: We own the lock and it is fresh.
     ours = 2
+    #: We own the lock but it has expired.  Another process trying
+    #: to acquire the lock will break it.
     ours_expired = 3
+    #: We don't own the lock; the hostname in the details matches our
+    #: hostname and there is no pid running that matches pid.  Therefore,
+    #: the lock is stale.
     stale = 4
+    #: Some other process owns the lock (probably) but it has expired.  Another
+    #: process trying to acquire the lock will break it.
     theirs_expired = 5
+    #: We don't own the lock; either our hostname does not match the
+    #: details, or there is a process (that's not us) with a matching pid.
+    #: The state of the lock is therefore unknown.
     unknown = 6
 
 
@@ -148,18 +156,19 @@ class Lock:
 
     :param lockfile: The full path to the lock file.
     :param lifetime: The expected maximum lifetime of the lock, as a
-        timedelta or integer number of seconds.  Defaults to 15 seconds.
+        timedelta or integer number of seconds, relative to now.  Defaults
+        to 15 seconds.
     :param separator: The separator character to use in the lock file's
         file name.  Defaults to the vertical bar (`|`) on POSIX systems
         and caret (`^`) on Windows.
-        """
+    """
 
     def __init__(
-            self,
-            lockfile: str,
-            lifetime: Optional[Interval] = None,
-            separator: str = SEP
-            ):
+        self,
+        lockfile: str,
+        lifetime: Optional[Interval] = None,
+        separator: str = SEP,
+    ):
         """Create the resource lock using the given file name and lifetime."""
         # The hostname has to be defined before we call _set_claimfile().
         self._hostname = socket.getfqdn()
@@ -172,8 +181,6 @@ class Lock:
         self._separator = separator
         self._claimfile: str
         self._set_claimfile()
-        # For transferring ownership across a fork.
-        self._owned = True
         # For extending the set of NFS errnos that are retried in _read().
         self._retry_errnos: List[int] = []
 
@@ -182,7 +189,10 @@ class Lock:
             self.__class__.__name__,
             self._lockfile,
             ('locked' if self._is_locked_no_refresh() else 'unlocked'),
-            self._lifetime, os.getpid(), id(self))
+            self._lifetime,
+            os.getpid(),
+            id(self),
+        )
 
     @property
     def hostname(self) -> str:
@@ -210,7 +220,8 @@ class Lock:
         # Rearrange for signature.
         try:
             lockfile, hostname, pid, random_ignored = filename.split(
-                self._separator)
+                self._separator
+            )
         except ValueError as error:
             raise NotLockedError('Details are unavailable') from error
         return hostname, int(pid), lockfile
@@ -225,7 +236,8 @@ class Lock:
             return LockState.unlocked
         try:
             lockfile, hostname, pid_str, random_ignored = filename.split(
-                self._separator)
+                self._separator
+            )
             pid = int(pid_str)
         except (ValueError, TypeError):
             # The contents of the lock file is corrupt, so we can't know
@@ -243,7 +255,7 @@ class Lock:
 
     @property
     def lifetime(self) -> timedelta:
-        "The current lock life time."""
+        """The current lock life time."""
         return self._lifetime
 
     @lifetime.setter
@@ -253,16 +265,22 @@ class Lock:
         else:
             self._lifetime = timedelta(seconds=lifetime)
 
-    def refresh(self,
-                lifetime: Optional[Interval] = None,
-                *,
-                unconditionally: bool = False):
+    def refresh(
+        self,
+        lifetime: Optional[Interval] = None,
+        *,
+        unconditionally: bool = False
+    ) -> None:
         """Refreshes the lifetime of a locked file.
 
         Use this if you realize that you need to keep a resource locked longer
         than you thought.
 
-        :param lifetime: If given, this sets the lock's new lifetime.
+        :param lifetime: If given, this sets the lock's new lifetime.  It
+            represents the number of seconds into the future that the
+            lock's lifetime will expire, relative to now, or whenever it is
+            refreshed, either explicitly or implicitly.  If not given, the
+            original lifetime interval is used.
         :param unconditionally: When False (the default), a ``NotLockedError``
             is raised if an unlocked lock is refreshed.
         :raises NotLockedError: if the lock is not set, unless optional
@@ -337,8 +355,9 @@ class Lock:
                 elif self._linkcount != 2:
                     # Somebody's messin' with us!  Log this, and try again
                     # later.  XXX should we raise an exception?
-                    log.error('unexpected linkcount: {0:d}'.format(
-                        self._linkcount))
+                    log.error(
+                        'unexpected linkcount: {0:d}'.format(self._linkcount)
+                    )
                 elif self._read() == self._claimfile:
                     # It was us that already had the link.
                     log.debug('already locked: {}'.format(self._lockfile))
@@ -355,10 +374,12 @@ class Lock:
             # find out if the lock lifetime has expired.  Cache the release
             # time to avoid race conditions.  (LP: #827052)
             release_time = self._releasetime
-            if (release_time != -1 and
-                    datetime.now() > release_time + CLOCK_SLOP):
-                # Yes, so break the lock.
-                self._break()
+            if release_time != -1:
+                now = datetime.now()
+                future = cast(datetime, release_time) + CLOCK_SLOP
+                if now > future:
+                    # Yes, so break the lock.
+                    self._break()
                 log.error('lifetime has expired, breaking')
             # Okay, someone else has the lock, our claim hasn't timed out yet,
             # and the expected lock lifetime hasn't expired yet either.  So
@@ -422,88 +443,36 @@ class Lock:
             return False
         return self._is_locked_no_refresh()
 
-    def __enter__(self):
+    def __enter__(self) -> 'Lock':
         self.lock()
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
+    ) -> Literal[False]:
         self.unlock()
         # Don't suppress any exception that might have occurred.
         return False
 
-    def transfer_to(self, pid: int) -> None:
-        """Transfer ownership of the lock to another process.
-
-        Use this only if you're transfering ownership to a child process across
-        a fork.  Use at your own risk, but it should be race-condition safe.
-        transfer_to() is called in the parent, passing in the pid of the child.
-        take_possession() is called in the child, and blocks until the parent
-        has transferred possession to the child.  disown() is used to set the
-        'owned' flag to False, and it is a disgusting wart necessary to make
-        forced lock acquisition work. :(
-
-        :param pid: The process id of the child process that will take
-            possession of the lock.
-        """
-        # First touch it so it won't get broken while we're fiddling about.
-        self._touch()
-        # Get the current claim's file name.
-        winner = self._read()
-        # Now twiddle ours to the given pid.
-        self._set_claimfile(pid)
-        # Create a hard link from the global lock file to the claim file.
-        # This actually does things in reverse order of normal operation
-        # because we know that lockfile exists, and claimfile better not!
-        os.link(self._lockfile, self._claimfile)
-        # Now update the lock file to contain a reference to the new owner.
-        self._write()
-        # Toggle off our ownership of the file so we don't try to finalize it
-        # in our __del__().
-        self._owned = False
-        # Unlink the old winner, completing the transfer.
-        os.unlink(winner)
-        # And do some sanity checks
-        assert self._linkcount == 2, (
-            'Unexpected link count: wanted 2, got {0:d}'.format(
-                self._linkcount))
-        assert self.is_locked, 'Expected to be locked'
-        log.debug('transferred the lock: {}'.format(self._lockfile))
-
-    def take_possession(self) -> None:
-        """Take possession of a lock from another process.
-
-        See `transfer_to()` for more information.
-        """
-        self._set_claimfile()
-        # Wait until the linkcount is 2, indicating the parent has completed
-        # the transfer.
-        while self._linkcount != 2 or self._read() != self._claimfile:
-            time.sleep(0.25)
-        log.debug('took possession of the lock: {}'.format(self._lockfile))
-
-    def disown(self) -> None:
-        """Disown this lock.
-
-        See `transfer_to()` for more information.
-        """
-        self._owned = False
-
-    def _set_claimfile(self, pid: Optional[int] = None) -> None:
+    def _set_claimfile(self) -> None:
         """Set the _claimfile private variable."""
-        if pid is None:
-            pid = os.getpid()
         # Calculate a hard link file name that will be used to lay claim to
         # the lock.  We need to watch out for two Lock objects in the same
         # process pointing to the same lock file.  Without this, if you lock
         # lf1 and do not lock lf2, lf2.locked() will still return True.
-        self._claimfile = self._separator.join((
-            self._lockfile,
-            self.hostname,
-            str(pid),
-            str(random.randint(0, MAXINT)),
-            ))
+        self._claimfile = self._separator.join(
+            (
+                self._lockfile,
+                self.hostname,
+                str(os.getpid()),
+                str(random.randint(0, MAXINT)),
+            )
+        )
 
-    def _write(self):
+    def _write(self) -> None:
         """Write our claim file's name to the claim file."""
         # Make sure it's group writable.
         with open(self._claimfile, 'w') as fp:
@@ -515,12 +484,12 @@ class Lock:
         return self._retry_errnos[:]
 
     @retry_errnos.setter
-    def retry_errnos(self, errnos: List[int]):
+    def retry_errnos(self, errnos: List[int]) -> None:
         self._retry_errnos = []
         self._retry_errnos.extend(errnos)
 
     @retry_errnos.deleter
-    def retry_errnos(self):
+    def retry_errnos(self) -> None:
         self._retry_errnos = []
 
     @property
@@ -533,7 +502,7 @@ class Lock:
         """Return the lock file name."""
         return self._lockfile
 
-    def _read(self):
+    def _read(self) -> Optional[str]:
         """Read the contents of our lock file.
 
         :return: The contents of the lock file or None if the lock file does
@@ -551,7 +520,7 @@ class Lock:
                 else:
                     return None
 
-    def _touch(self, filename=None):
+    def _touch(self, filename: Optional[str] = None) -> None:
         """Touch the claim file into the future.
 
         :param filename: If given, the file to touch, otherwise our claim file
@@ -567,7 +536,7 @@ class Lock:
                 raise
 
     @property
-    def _releasetime(self):
+    def _releasetime(self) -> Union[int, datetime]:
         """The time when the lock should be released.
 
         :return: The mtime of the file, which is when the lock should be
@@ -581,7 +550,7 @@ class Lock:
             raise
 
     @property
-    def _linkcount(self):
+    def _linkcount(self) -> int:
         """The number of hard links to the lock file.
 
         :return: the number of hard links to the lock file, or -1 if the lock
@@ -594,7 +563,7 @@ class Lock:
                 return -1
             raise
 
-    def _break(self):
+    def _break(self) -> None:
         """Break the lock."""
         # First, touch the lock file.  This reduces but does not eliminate the
         # chance for a race condition during breaking.  Two processes could
@@ -613,7 +582,7 @@ class Lock:
         # Remove the global lockfile, which actually breaks the lock.
         try:
             os.unlink(self._lockfile)
-        except OSError as error:
+        except OSError as error:                    # pragma: nocover
             if error.errno not in ERRORS:
                 raise
         # Try to remove the old winner's claim file, since we're assuming the
@@ -621,13 +590,13 @@ class Lock:
         # unlink their claim file -- this doesn't wreck the locking algorithm,
         # but will leave claim file turds laying around, a minor inconvenience.
         try:
-            if winner:
+            if winner:                              # pragma: nobranch
                 os.unlink(winner)
         except OSError as error:
-            if error.errno not in ERRORS:
+            if error.errno not in ERRORS:           # pragma: nocover
                 raise
 
-    def _sleep(self):
+    def _sleep(self) -> None:
         """Snooze for a random amount of time."""
         interval = random.random() * 2.0 + 0.01
         time.sleep(interval)
